//----------------------------------------//
// d3d.cpp - the main file for the Direct3D
//			related functions
//
//----------------------------------------//

//include the globals
#include "d3d.h"


//----------------------------------------//
// Name: D3D()
// Purpose: initializes all member variables
//----------------------------------------//
D3D::D3D()
{
	//set all member variables to default values
	lpd3d = NULL;
	lpd3ddevice = NULL;

	ColorDepth = 0;
	MaxSimultaneousTextures = 0;
	MaxMultisample = 0;

	Windowed = true;
	ScreenWidth = 0;
	ScreenHeight = 0;

}

//----------------------------------------//
// Name: ~D3D()
// Purpose: the destructor
//----------------------------------------//
D3D::~D3D()
{
	//call the shutdown function automatically - just in case
	//it wasn't already called.  That way it is always shut down
	//and reduces the risk of a memory leak.  Calling the shutdown 
	//function more than once shouldn't hurt.
	//D3DShutdown(); *** temp***
}


//----------------------------------------//
// Name: D3DInitialize()
// Purpose: Initialize the main parts of 
//	Direct 3D such as the screen resolution, etc
// Note: the parameters are for fullscreen mode screen dimenstions
//----------------------------------------//
int D3D::D3DInitialize(HWND hWnd)
{
	D3DPRESENT_PARAMETERS d3dpp;	//create the present parameters variable
	D3DDISPLAYMODE d3ddm;			//the D3D current display mode variable 
	D3DCAPS8 pCaps;				//pointer to the device caps


	logfile.LogtoFile("D3D - D3DInitialize() called\n");

	//the variable to see if it can run hardware vertex processing
	bool HardwareVertexProcessing	= false;
	//the variables to see if it can run 5-6-5 mode
	bool Mode565					= false;
	bool ModeX555					= false;
	bool ModeA555					= false;

	bool Bitdepth32X				= false; //test for X8R8G8B8 bit color
	bool Bitdepth32A				= false; //test for A8R8G8B8 bit color

	// Create the D3D object
    if((lpd3d = Direct3DCreate8(D3D_SDK_VERSION))== NULL)
	{
		logfile.LogtoFile("D3D - Failed to create a Direct3D 8 object\n");
        return ENGINE_BAD;
	}

	//try reading in some the the device capabilities
	if(FAILED(lpd3d->GetDeviceCaps(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, &pCaps)))
	{
		logfile.LogtoFile("D3D - Unable to read in Device Capabilities\n");
		return ENGINE_BAD;
	}

	//maximum simultaneous textures - good to know because it determines whether 
	//some of the special effects like gloss mapping, etc can or can't be used
	MaxSimultaneousTextures = pCaps.MaxSimultaneousTextures;
	logfile.outfile<<"D3D - Max Number of Simultaneous textures is "<<MaxSimultaneousTextures<<"\n";

	//test for hardware vertex processing
	if(pCaps.DevCaps & D3DDEVCAPS_HWTRANSFORMANDLIGHT)
	{
		//looks like it supports hardware tnl
		HardwareVertexProcessing = true;
		logfile.LogtoFile("D3D - Device appears to support hardware vertex processing.\n");
	} else
		HardwareVertexProcessing = false;

	//clear the present parameters and start setting them up
	ZeroMemory(&d3dpp, sizeof(d3dpp));

	//if the config file said it was windowed, do it in windowed mode,
	//otherwise do it in fullscreen with the resolution that it said
	if(Windowed == true)
	{
		//log to file - windowed mode selected
		logfile.LogtoFile("D3D - Windowed mode selected \n");

		// Get the current desktop display mode, so we can set up a back
		// buffer of the same format
		if( FAILED( lpd3d->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &d3ddm)))
		{
			logfile.LogtoFile("D3D - Failed to get the display adapter mode \n");
			return ENGINE_BAD;
		}

		d3dpp.Windowed = TRUE;
		d3dpp.BackBufferCount = 1;
		d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
		d3dpp.BackBufferFormat = d3ddm.Format;
		d3dpp.BackBufferWidth = ScreenWidth;
		d3dpp.BackBufferHeight = ScreenHeight;
		d3dpp.EnableAutoDepthStencil = TRUE;
		d3dpp.AutoDepthStencilFormat = D3DFMT_D16;

		//check if multisampling should be used
		if(MaxMultisample > 0)
		{
			if(MaxMultisample == 2)
			{
				if(SUCCEEDED(lpd3d->CheckDeviceMultiSampleType( D3DADAPTER_DEFAULT, 
					D3DDEVTYPE_HAL , d3ddm.Format, TRUE, D3DMULTISAMPLE_2_SAMPLES)))
				{
					//full-scene antialiasing is supported. Enable it here.
					d3dpp.MultiSampleType = D3DMULTISAMPLE_2_SAMPLES;
					logfile.LogtoFile("D3D - Using 2 multisamples.\n");
				}
			}

			if(MaxMultisample == 4)
			{
				if(SUCCEEDED(lpd3d->CheckDeviceMultiSampleType( D3DADAPTER_DEFAULT, 
					D3DDEVTYPE_HAL , d3ddm.Format, TRUE, D3DMULTISAMPLE_4_SAMPLES)))
				{
					//full-scene antialiasing is supported. Enable it here.
					d3dpp.MultiSampleType = D3DMULTISAMPLE_4_SAMPLES;
					logfile.LogtoFile("D3D - Using 4 multisamples.\n");
				}
			}

		}

		//check the device caps to see if hardware TnL is supported.  If it is, use
		//it!
		if(HardwareVertexProcessing == true)
		{
			//try using hardware vertex processing
			if(FAILED(lpd3d->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
                                      D3DCREATE_HARDWARE_VERTEXPROCESSING,
                                      &d3dpp, &lpd3ddevice)))
			{
				//looks like it failed for some reason
				logfile.LogtoFile("D3D - Failed to use windowed hardware vertex processing\n");
				return ENGINE_BAD;
			} else
				logfile.LogtoFile("D3D - Succesfully created a windowed hardware vertex processing device\n");

		} else {
			//try using software vertex processing because it looks like hardware
			//isn't available
			if(FAILED(lpd3d->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
                                      D3DCREATE_SOFTWARE_VERTEXPROCESSING,
                                      &d3dpp, &lpd3ddevice)))
			{
				logfile.LogtoFile("D3D - Failed to use windowed software vertex processing\n");
				return ENGINE_BAD;
			} else
				logfile.LogtoFile("D3D - Succesfully created a windowed software vertex processing device\n");

		} //end if for caps
	
	} else {

		//logfile - fullscreen mode selected
		logfile.LogtoFile("D3D - Fullscreen mode selected \n");

		//run through all the color formats and find the best one

		//check to see if it supports D3DFMT_A8R8G8B8
		if (lpd3d->CheckDeviceType(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, D3DFMT_A8R8G8B8, 
			D3DFMT_X8R8G8B8, FALSE)==D3D_OK)
		{
			logfile.LogtoFile("D3D - D3DFMT_A8R8G8B8 supported \n");
			Bitdepth32A = true;
		}

		//check to see if it supports D3DFMT_X8R8G8B8
		if (lpd3d->CheckDeviceType(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, D3DFMT_X8R8G8B8, 
				D3DFMT_X8R8G8B8, FALSE)==D3D_OK)
		{
			logfile.LogtoFile("D3D - D3DFMT_X8R8G8B8 supported \n");
			Bitdepth32X = true;
		}

		//check to see if it supports D3DFMT_R5G6B5
		if (lpd3d->CheckDeviceType(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, D3DFMT_R5G6B5, 
				D3DFMT_R5G6B5, FALSE)==D3D_OK)
		{
			logfile.LogtoFile("D3D - D3DFMT_R5G6B5 supported \n");
			Mode565= true;
		}

		//check to see if it supports D3DFMT_X1R5G5B5
		if (lpd3d->CheckDeviceType(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, 
				D3DFMT_X1R5G5B5, D3DFMT_X1R5G5B5, FALSE)==D3D_OK)
		{
			logfile.LogtoFile("D3D - D3DFMT_X1R5G5B5 supported \n");
			ModeX555 = true;
		}

		//check to see if it supports D3DFMT_A1R5G5B5
		if (lpd3d->CheckDeviceType(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, 
				D3DFMT_A1R5G5B5, D3DFMT_X1R5G5B5, FALSE)==D3D_OK)
		{
			logfile.LogtoFile("D3D - D3DFMT_A1R5G5B5 supported \n");
			ModeA555 = true;
		}

		//log it to the file that a backbuffer format can't be found
		if((Bitdepth32X | Bitdepth32A | Mode565 | ModeA555 | ModeX555)==false)
		{
			logfile.LogtoFile("D3D - Unable to find a supported backbuffer format \n");
			return ENGINE_BAD;
		}

	

		//check if it should be using 16 bit color or 32 bit color according
		//to the logfile.
		if(ColorDepth == 16)
		{
			//test for A555
			if (ModeA555 == true)
				d3dpp.BackBufferFormat = D3DFMT_A1R5G5B5;

			//test for X555
			if (ModeX555 == true)
				d3dpp.BackBufferFormat = D3DFMT_X1R5G5B5;

			//test for 565
			if (Mode565 == true)
				d3dpp.BackBufferFormat = D3DFMT_R5G6B5;
		}

		//test the colordepth variable for 32 bit color
		if(ColorDepth == 32)
		{
			//test for 32A second last
			if(Bitdepth32A == true)
				d3dpp.BackBufferFormat = D3DFMT_A8R8G8B8;

			//test for 32X last
			if (Bitdepth32X == true)
				d3dpp.BackBufferFormat = D3DFMT_X8R8G8B8;
		}

		d3dpp.Windowed = FALSE;
		d3dpp.SwapEffect = D3DSWAPEFFECT_FLIP;
		d3dpp.BackBufferWidth = ScreenWidth;
		d3dpp.BackBufferHeight = ScreenHeight;
		d3dpp.BackBufferCount = 1;
		d3dpp.FullScreen_RefreshRateInHz = D3DPRESENT_RATE_DEFAULT;
		d3dpp.FullScreen_PresentationInterval = D3DPRESENT_INTERVAL_ONE;
		d3dpp.hDeviceWindow = hWnd;
		d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
		d3dpp.EnableAutoDepthStencil = TRUE;

		//check if multisampling should be used
		if(MaxMultisample > 0)
		{
			if(MaxMultisample == 2)
			{
				if(SUCCEEDED(lpd3d->CheckDeviceMultiSampleType( D3DADAPTER_DEFAULT, 
					D3DDEVTYPE_HAL , d3dpp.BackBufferFormat, FALSE, D3DMULTISAMPLE_2_SAMPLES)))
				{
					//full-scene antialiasing is supported. Enable it here.
					d3dpp.MultiSampleType = D3DMULTISAMPLE_2_SAMPLES;
					logfile.LogtoFile("D3D - Using 2 multisamples.\n");
				}
			}

			if(MaxMultisample == 4)
			{
				if(SUCCEEDED(lpd3d->CheckDeviceMultiSampleType( D3DADAPTER_DEFAULT, 
					D3DDEVTYPE_HAL , d3dpp.BackBufferFormat, FALSE, D3DMULTISAMPLE_4_SAMPLES)))
				{
					//full-scene antialiasing is supported. Enable it here.
					d3dpp.MultiSampleType = D3DMULTISAMPLE_4_SAMPLES;
					logfile.LogtoFile("D3D - Using 4 multisamples.\n");
				}
			}

		}

		//now test to see if its in 16 bit color mode and use a 16bit z buffer
		if(((Mode565 | ModeA555 | ModeX555)==true))
		{
			logfile.LogtoFile("D3D - 16 bit z-buffer and backbuffer supported \n");
			d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
		}

		//test if its 32 bit color, and if it is, try to use a 32 bit z buffer
		if((Bitdepth32X | Bitdepth32A)==true)
		{
			logfile.LogtoFile("D3D - 32 bit z-buffer and backbuffer supported \n");
			logfile.LogtoFile("D3D - Using 32 bit backbuffer and z-buffer \n");

			d3dpp.AutoDepthStencilFormat = D3DFMT_D32;

		}
			d3dpp.AutoDepthStencilFormat = D3DFMT_D16;

		//check the device caps to see if hardware TnL is supported.  If it is,
		//use it, and if not, drop back to software vertex processing
		if(HardwareVertexProcessing == true)
		{
			//it appears hardware TnL is there.  Use it!
			if(lpd3d->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd, 
				D3DCREATE_HARDWARE_VERTEXPROCESSING, &d3dpp, &lpd3ddevice)!= D3D_OK)
			{
				//it failed for one reason or another
				logfile.LogtoFile("D3D - Failed to create a fullscreen hardware vertex processing device\n");
				return ENGINE_BAD;
			} else
				logfile.LogtoFile("D3D - Succesfully created a fullscreen hardware vertex processing device\n");

		} else {
			//it appears no hardware TnL is there.  Use software vertex processing
			if(lpd3d->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
				D3DCREATE_SOFTWARE_VERTEXPROCESSING, &d3dpp, &lpd3ddevice)!=D3D_OK)
			{
				logfile.LogtoFile("D3D - Failed to create a fullscreen software vertex processing device\n");
				return ENGINE_BAD;
			} else
				logfile.LogtoFile("D3D - Succesfully created a fullscreen software vertex processing device\n");

		}

	} //end big if statement for windowedmode


	//set up the default render states
	lpd3ddevice->SetRenderState(D3DRS_DITHERENABLE, TRUE);	//dithering
	lpd3ddevice->SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);	//z buffer enabled...

	//trilinear filtering...
	lpd3ddevice->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
	lpd3ddevice->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
	lpd3ddevice->SetTextureStageState(0, D3DTSS_MIPFILTER, D3DTEXF_LINEAR);

	//turn on multisampling if its supposed to be
	if(MaxMultisample > 0)
	{
		lpd3ddevice->SetRenderState(D3DRS_MULTISAMPLEANTIALIAS, TRUE);
	}

	//z buffer enabled...
	lpd3ddevice->SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);

	// Create the light. This is basically a normal white light that's hanging above
	// and slightly in back of the model (you can look at the position values.)
	D3DLIGHT8 d3dLight;
	ZeroMemory(&d3dLight, sizeof(D3DLIGHT8));
	d3dLight.Type = D3DLIGHT_DIRECTIONAL;
	d3dLight.Diffuse.r = 0.20f;
	d3dLight.Diffuse.g = 0.20f;
	d3dLight.Diffuse.b = 0.80f;
	d3dLight.Specular.r = 1.0f;
	d3dLight.Specular.g = 1.0f;
	d3dLight.Specular.b = 1.0f;
	d3dLight.Position.x = 0.0f;
	d3dLight.Position.y = 1000.0f;
	d3dLight.Position.z = 0.0f;
	d3dLight.Direction.x = 1.0f;
	d3dLight.Direction.y = -1.0f;
	d3dLight.Direction.z = -1.0f;
	lpd3ddevice->SetLight(0, &d3dLight);

	D3DLIGHT8 pointlight;
	ZeroMemory(&pointlight, sizeof(D3DLIGHT8));
	pointlight.Type = D3DLIGHT_POINT;
	pointlight.Diffuse.r = 1.0f;
	pointlight.Diffuse.g = 0.6f;
	pointlight.Diffuse.b = 0.25f;
	pointlight.Specular.r = 1.0f;
	pointlight.Specular.g = 0.5f;
	pointlight.Specular.b = 0.25f;
	pointlight.Position.x = 0.0f;
	pointlight.Position.y = 2.0f;
	pointlight.Position.z = 0.0f;
	pointlight.Direction.x = 0.0f;
	pointlight.Direction.y = 0.0f;
	pointlight.Direction.z = 0.0f;
	pointlight.Range = 2.5f;
	//pointlight.Falloff = 1.0f;
	pointlight.Attenuation0 = 1.0f;
	pointlight.Attenuation1 = 0.0f;
	lpd3ddevice->SetLight(1, &pointlight);

	lpd3ddevice->LightEnable(0, TRUE);
	lpd3ddevice->LightEnable(1, TRUE);

	lpd3ddevice->SetRenderState(D3DRS_LIGHTING, TRUE);
	lpd3ddevice->SetRenderState(D3DRS_SPECULARENABLE, TRUE);
	lpd3ddevice->SetRenderState( D3DRS_AMBIENT, 0xffffffff );

	//log that its all ok, and return ok
	logfile.LogtoFile("D3D - Initialization succesful. \n");
	return(D3D_OK);

}


//----------------------------------------//
// Name: D3DShutdown()
// Purpose: release all the main d3d resources
//----------------------------------------//
void D3D::D3DShutdown()
{
	//log it
	logfile.LogtoFile("D3D Shutdown called. \n");

	//safely release the pointer thingies
	SAFE_RELEASE(lpd3ddevice);
	SAFE_RELEASE(lpd3d);

}
