//----------------------------------------//
// engine.cpp - the main file for the main 
//		engine related functions
//
//----------------------------------------//

//include the globals, as usual
#include "engine.h"

//----------------------------------------//
// Name: ENGINE()
// Purpose: initializes GameState setting
//----------------------------------------//
ENGINE::ENGINE()
{
	//set the gamestate to start the program
	GameState = RENDER_MAIN_MENU;	//*********

	//give default values to the variables
	ShowFPS = false;

}

//----------------------------------------//
// Name: ~ENGINE()
// Purpose: destructor - calls the shutdown function 
//----------------------------------------//
ENGINE::~ENGINE()
{
	Shutdown(); 
} 


//----------------------------------------//
// Name: MainLoop()
// Purpose: selects the correct function based
//		on what the gamestate variable is
//----------------------------------------//
int ENGINE::MainLoop()
{
	//update the timer stuff
	timer.UpdateTimer();

	if(timer.CheckTimer() == TIMER_OK)
	{
		//get the input no matter what the state - the input is handled for
		//every state inside the MainInputHandler function
		HRESULT result = MainInputHandler();

		if(result == ENGINE_BAD)
		{
			logfile.LogtoFile("ENG - input handler has failed, now quitting\n");
			return ENGINE_BAD;
		}

		if(result == ENGINE_QUIT)
		{
			logfile.LogtoFile("ENG - GameState set to Quit by InputHandler.  Now quitting...\n");
			return ENGINE_QUIT;
		}
	}

	//the main loop.  let 'er rip!
	switch(GameState)
	{
		case INTRO:
				//call the intro movie function - movie or scripted sequence
				break;
		case RENDER_MAIN_MENU:
				//call the functions for the game menu
				if(RenderMainMenu() != ENGINE_OK)
				{
					logfile.LogtoFile("ENG - RenderMainMenu failed.\n");
					return ENGINE_BAD;

				}

				break;
		case RENDER_MAIN_SCENE:
				//call the main rendering sequence
				if(RenderMainScene()==ENGINE_BAD)
				{
					logfile.LogtoFile("ENG - RenderMainScene failed.\n");
					return ENGINE_BAD;
				}

				break;
		case INGAMEMENU:
				//call the default function, the rendering loop
				break;
		case SHUTDOWN:
				//it isn't necessary to call the shutdown functions because the
				//destructor calls it, so all we do is tell it to quit running the 
				//main loop
				
				//return bad so that the main loop quits and the program ends
				return ENGINE_QUIT;
				break;
	} //end switch

	//return ok
	return ENGINE_OK;
	
	//return ok
	return ENGINE_OK;
}

//----------------------------------------//
// Name: ReadConfigurationFile()
// Purpose: Reads in all the info from the configuration file. 
// Note: It is necessary to read the information in before everything else is
//		initialized so that everything gets initialized with the right values.  
//----------------------------------------//
int ENGINE::ReadConfigurationFile(char *ConfigFileName)
{
	//load in the configuration file stuff next
	//open the configuration file
	FILE * pFile = fopen(ConfigFileName, "r");

	//check to see if it opened
	if(pFile == NULL)
		return ENGINE_BAD;
	else
		logfile.LogtoFile("ENG - Configuration file succesfully opened\n");

	//check to make sure its the right file
	if(fscanf(pFile, " //Configuration file ") == EOF)
	{
		logfile.LogtoFile("ENG - Failed to verify startup file type\n");
		return ENGINE_BAD;
	} else
		logfile.LogtoFile("ENG - Configuration file is of correct type\n");

	//try reading in the windowed mode variable from the file
	if(fscanf(pFile, "windowed %d ", &d3d.Windowed) != 1)
	{
		logfile.LogtoFile("ENG - Failed to load windowed mode variable\n");
		return ENGINE_BAD;
	} else
		logfile.outfile<<"ENG - Windowed mode read: "<<d3d.Windowed<<"\n";

	//try reading in the resolution
	if(fscanf(pFile, "resolution %d %d ",  &d3d.ScreenWidth, &d3d.ScreenHeight) != 2)
	{
		logfile.LogtoFile("ENG - Failed to load screen width from file\n");
		return ENGINE_BAD;
	} else
		logfile.outfile<<"ENG - Resolution set to "<<d3d.ScreenWidth<<"x"<<d3d.ScreenHeight<<"\n";

	//try reading in the color depth
	if(fscanf(pFile, "colordepth %d ", &d3d.ColorDepth) != 1)
	{
		logfile.LogtoFile("ENG - Failed to load color depth\n");
		return ENGINE_BAD;
	} else
		logfile.outfile<<"ENG - Color depth read in as: "<<d3d.ColorDepth<<"\n";

	//read in the level of multisampling to be used (if any)
	if(fscanf(pFile, "multisample %d ", &d3d.MaxMultisample) != 1)
	{
		logfile.LogtoFile("ENG - Failed to read in multisampling level.\n");
		return ENGINE_BAD;
	} else
		logfile.outfile<<"ENG - Multisample level read in as "<<d3d.MaxMultisample<<"\n";

	//read in whether to show the fps or not
	if(fscanf(pFile, "showfps %d ", &ShowFPS) != 1)
	{
		logfile.LogtoFile("ENG - Unable to load showFPS\n");
		return ENGINE_BAD;
	} else
		logfile.outfile<<"ENG - ShowFPS set to "<<ShowFPS<<"\n";

	//close the file
	fclose(pFile);
	logfile.LogtoFile("ENG - Configuration file closed\n");


	//must have gone ok...
	return ENGINE_OK;
}

//----------------------------------------//
// Name: HandleInput()
// Purpose: handles the input for engine
// Note: its here so that its easier to do stuff with the input
//----------------------------------------//
int ENGINE::MainInputHandler()
{
	//check what state the GameState is, and do the right input depending on what it is
	//make sure the keyboard is active
	directinput.ActivateKeyboard();

	//first try to read the keys, if they can't be read then don't sweat it - the
	//window might not be active, so its not a big deal
	if(directinput.ReadKeys() == INPUT_BAD)
	{
		//its not abig deal
		return ENGINE_OK;
	}
	
	//read the mouse in just like the keyboard
	if(directinput.ReadMouse() == INPUT_FAILEDREAD)
	{
		return ENGINE_BAD;
	}

	switch(GameState)
	{
		case RENDER_MAIN_MENU:
			//handle the input that should happen when the game 
			//is at the main menu
	
			//***temp***  -add in messagebox asking yes or no
			//if the escape key is down, exit the program
			if(directinput.KeyDown(DIK_ESCAPE))
			{
				return ENGINE_QUIT;
			}

			if(directinput.KeyDown(DIK_RETURN))
			{
				GameState = RENDER_MAIN_SCENE;
			}

			break;

		case INGAMEMENU:
			//hanld the input that should happen when the game is at
			//the options menu
			
			break;

		case RENDER_MAIN_SCENE:
			//hanlde the input for when the game is at the main rendering 
			//state

			//handle the basic movement and move the camera the right way depending
			//on what key was pressed
			if(directinput.KeyDown(DIK_LEFT))
			{
				//input.CameraTranslate(-0.05f, 0.0f, 0.0f);
				Camera.TranslateCamera(-2.0f, 0.0f, 0.0f);
			}

			if(directinput.KeyDown(DIK_A))
				Camera.TranslateCamera(-2.0f, 0.0f, 0.0f);

			if(directinput.KeyDown(DIK_RIGHT))
			{
				//input.CameraTranslate(0.05f, 0.0f, 0.0f);
				Camera.TranslateCamera(2.0f, 0.0f, 0.0f);
			}
			if(directinput.KeyDown(DIK_D))
				Camera.TranslateCamera(2.0f, 0.0f, 0.0f);

			if(directinput.KeyDown(DIK_UP))
			{
				//input.CameraTranslate(0.0f, 0.0f, 0.05f);
				Camera.TranslateCamera(0.0f, 0.0f, 2.0f);
			}

			if(directinput.KeyDown(DIK_W))
				Camera.TranslateCamera(0.0f, 0.0f, 2.0f);
			
			if(directinput.KeyDown(DIK_DOWN))
			{
				//input.CameraTranslate(0.0f, 0.0f, -0.05f);
				Camera.TranslateCamera(0.0f, 0.0f, -2.0f);
			}

			if(directinput.KeyDown(DIK_S))
				Camera.TranslateCamera(0.0f, 0.0f, -2.0f);

			//read in the mouse coordinates and use it to do the mouselook
			//new mouse coordinates
			float x = 0;
			float y = 0;
			x += (float)directinput.mousestate.lX; //get the x position from the mouse
			y += (float)directinput.mousestate.lY; //get the y position from the mouse

			//rotate the camera for the mouse look
			Camera.RotateCamera(x, y);

			//*** temp ***
			if(directinput.KeyDown(DIK_ESCAPE))
			{
				GameState = RENDER_MAIN_MENU;
			}

			break;

	} //end switch

	// return success
	return ENGINE_OK;;

}

//----------------------------------------//
// Name: Initialize()
// Purpose: initialize all the game related stuff
// Note: the parameter is necessary for Direct Input
//----------------------------------------//
int ENGINE::Initialize(HINSTANCE hInst, HWND hwnd)
{
	//set the engines main window handle to that of the program
	hWnd = hwnd;

	//initialize the timer
	if(timer.InitTimer() != TIMER_OK)
	{
		logfile.LogtoFile("ENG - timer.InitTimer() failed.\bn");
		return ENGINE_BAD;
	} else
		logfile.LogtoFile("ENG - timer.InitTimer() was successful.\n");

	//initialize Direct 3D next...
	if(d3d.D3DInitialize(hWnd) != ENGINE_OK)
	{
		logfile.LogtoFile("ENG - d3d.D3DInitialize() failed.\n");
		return ENGINE_BAD;
	} else
		logfile.LogtoFile("ENG - d3d.D3DInitialize() was successful.\n");

	//initialize Direct Input
	if(directinput.InitializeDirectInput(hInst, hWnd) != INPUT_OK)
	{
		logfile.LogtoFile("ENG - InitializeDirectInput failed.\n");
		return ENGINE_BAD;
	} else
		logfile.LogtoFile("ENG - InitializeDirectInput was successful.\n");

	//initialize the camera
	Camera.InitializeCamera(d3d.lpd3ddevice, d3d.ScreenWidth, d3d.ScreenHeight);

	//initialize the less important stuff


	//the font
	font.InitializeFont(d3d.lpd3ddevice, 12);
	font.SetFontColor(128, 255, 255, 255);

	//the spinning 4E3 logo
	if(FourE3.CreateModelFromFile(d3d.lpd3ddevice, 
		"data\\objects\\4e3\\4e3.txt", 1.0f) != MSMODEL_SUCCESS)
	{
		logfile.LogtoFile("ENG - model not loaded.\n");
		return ENGINE_BAD;
	} else 
		logfile.LogtoFile("ENG - model loaded.\n");

	//load the level
	if(Level.CreateModelFromFile(d3d.lpd3ddevice, 
		"data\\objects\\level\\level.txt", 1.0f) != MSMODEL_SUCCESS)
	{
		logfile.LogtoFile("ENG - level not loaded.\n");
		return ENGINE_BAD;
	} else 
		logfile.LogtoFile("ENG - level loaded.\n");

	//load the skybox
	if(Skybox.CreateModelFromFile(d3d.lpd3ddevice, 
		"data\\objects\\skybox\\skybox.txt", 1.0f) != MSMODEL_SUCCESS)
	{
		logfile.LogtoFile("ENG - skybox not loaded.\n");
		return ENGINE_BAD;
	} else 
		logfile.LogtoFile("ENG - skybox loaded.\n");

	//load the cloud box
	if(Cloudbox.CreateModelFromFile(d3d.lpd3ddevice, 
		"data\\objects\\cloud box\\cloud box.txt", 1.0f) != MSMODEL_SUCCESS)
	{
		logfile.LogtoFile("ENG - cloudbox not loaded.\n");
		return ENGINE_BAD;
	} else 
		logfile.LogtoFile("ENG - cloudbox loaded.\n");


	if(MenuSprite1.CreateSprite(d3d.lpd3ddevice, "data\\textures\\view the entry.bmp")!= GUI_OK)
	{
		logfile.LogtoFile("ENG - sprite not created.\n");
		return ENGINE_BAD;
	} else 
		logfile.LogtoFile("ENG - sprite created.\n");

	if(MenuSprite2.CreateSprite(d3d.lpd3ddevice, "data\\textures\\escape.bmp")!= GUI_OK)
	{
		logfile.LogtoFile("ENG - sprite not created.\n");
		return ENGINE_BAD;
	} else 
		logfile.LogtoFile("ENG - sprite created.\n");


	if(TestBillboard1.InitializeBillboard(d3d.lpd3ddevice, "data\\textures\\particle.bmp", 
				3.0f, D3DXVECTOR3(0.0f, 2.0f, 0.0f)) != BILLBOARD_OK)
	{
		logfile.LogtoFile("ENG - billboard not created.\n");
		return ENGINE_BAD;
	} else 
		logfile.LogtoFile("ENG - billboard created.\n");

	if(TestBillboard2.InitializeBillboard(d3d.lpd3ddevice, "data\\textures\\flame.bmp", 
				0.3f, D3DXVECTOR3(0.0f, 1.85f, 0.0f)) != BILLBOARD_OK)
	{
		logfile.LogtoFile("ENG - billboard not created.\n");
		return ENGINE_BAD;
	} else 
		logfile.LogtoFile("ENG - billboard created.\n");

	
	//return ok
	return ENGINE_OK;
}


//----------------------------------------//
// Name: Shutdown()
// Purpose: shut down all the stuff for the game
// Note: things must be released in the oposite order
//		that they were allocated, otherwise things go bad
//----------------------------------------//
void ENGINE::Shutdown()
{
	//shut down the stuff that was initialized
	d3d.D3DShutdown();

	directinput.ShutdownDirectInput();
	

}

