//----------------------------------------//
// gui.cpp - main file for graphical user
//			interface functions
//
//----------------------------------------//

//include the header file
#include "gui.h"

//the sprite class functions

//--------------------------------------------------//
// Name: ~Sprite()
// Desc: destructor - automatically de-allocates everything
//--------------------------------------------------//
Sprite::~Sprite()
{
	//simply destroy the sprite
	DestroySprite();

}


//--------------------------------------------------//
// Name: CreateSprite()
// Desc: the function that creates the sprite 
//--------------------------------------------------//
bool Sprite::CreateSprite(LPDIRECT3DDEVICE8 device, LPCSTR filename)
{
	Device = device;

	//load the texture
	D3DXCreateTextureFromFileExA(Device, //device
			filename,			//source file
			D3DX_DEFAULT,		//width
			D3DX_DEFAULT,		//height
			0,					//mip levels
			0,					//usage (not rendering to texture)
			D3DFMT_UNKNOWN,		//image format - not known
			D3DPOOL_MANAGED,	//managed memory
			D3DX_DEFAULT,		//filtering - default
			D3DX_DEFAULT,		//mip filtering - default
			0xff000000,			//color key
			NULL,				//source info
			NULL,				//palette info
			&texture);			//the texture


	
	//now create the sprite
	D3DXCreateSprite(Device, &lpd3dxsprite);

	//return ok
	return GUI_OK;
}


//--------------------------------------------------//
// Name: DrawSprite()
// Desc: function that draws the sprite
//--------------------------------------------------//
void Sprite::DrawSprite(D3DXVECTOR2 pos)
{
	//begin drawing
	lpd3dxsprite->Begin();

	//draw the sprite - sub the temporary structures into the function
	lpd3dxsprite->Draw(texture, //the texture parameter
		NULL,			 //the source rectangle of the texture
		NULL,			 //the scaling vector
		NULL,			 //rotation center - a D3DXVECTOR2 structure
		0,				 //value of the rotation
		&pos,			 //translation vector - a D3DXVECTOR2 structure -aka position
		0xffffffff);	 //the color ***temp***

	//stop drawing
	lpd3dxsprite->End();

}

//--------------------------------------------------//
// Name: DestroySprite()
// Desc: function that de-allocates all the stuff for the sprites
//--------------------------------------------------//
void Sprite::DestroySprite()
{

	//release the texture
	if(texture!=NULL)
		 texture->Release();

	//release the lpdedxsprite
	if(lpd3dxsprite!=NULL)
		lpd3dxsprite->Release();

}

/*//--------------------------------------------------//
// Name: Sprite()
// Desc: constructor - initializes internal variables
//--------------------------------------------------//
Sprite::Sprite()
{
	//give all the variables default values
	Color = D3DCOLOR_ARGB(255, 255, 255, 255);
	Device = NULL;
	Height = 0.0f;
	MouseOver = false;
	Position = D3DXVECTOR2(0.0f, 0.0f);
	ScreenHeight = 0.0f;
	ScreenWidth = 0.0f;
	Tex = NULL;
	TexOver = NULL;
	VertexBuffer = NULL;
	Width = 0.0f;
}

//--------------------------------------------------//
// Name: ~Sprite()
// Desc: destructor - makes sure things are deallocated
//--------------------------------------------------//
Sprite::~Sprite()
{
	//simply call the destroy function
	DestroySprite();
}

//--------------------------------------------------//
// Name: CreateSprite()
// Desc: creates a sprite, what else?
//--------------------------------------------------//
int Sprite::CreateSprite(LPDIRECT3DDEVICE8 device, LPCSTR texname, 
					LPCSTR texovername, float width, float height, D3DXVECTOR2 pos, 
					float screenwidth, float screenheight)
{
	//set the device first
	Device = device;

	//set the other variables to what they should be
	Width = width;
	Height = height;
	Position.x = pos.x;
	Position.y = pos.y;
	ScreenWidth = screenwidth;
	ScreenHeight = screenheight;


	//try loading in the first texture - the one that will be shown no matter if the
	//mouse is over the sprite or not
	if(FAILED(D3DXCreateTextureFromFileExA(Device,	//d3d device 
		texname,		//filename
		D3DX_DEFAULT,	//width
		D3DX_DEFAULT,	//height
		D3DX_DEFAULT,	//mip levels
		0,				//usage - not a render target
		D3DFMT_UNKNOWN,	//format
		D3DPOOL_DEFAULT, //memory pool
		D3DX_DEFAULT, //filtering
		D3DX_DEFAULT, //mipmap filtering
		0xff000000,	//colorkey
		NULL,		//source info
		NULL,		//palette
		&Tex)))
	{
		//looks like it failed, which is a bad thing
		logfile.LogtoFile("SPR - failed to load basic sprite texture.\n");
		//return GUI_BAD;
	}

	//try loading in the mouseover texture, if there is supposed to be one
	if(texovername != NULL)
	{
		if(FAILED(D3DXCreateTextureFromFileExA(Device,	//d3d device 
			texovername,		//filename
			D3DX_DEFAULT,	//width
			D3DX_DEFAULT,	//height
			D3DX_DEFAULT,	//mip levels
			0,				//usage - not a render target
			D3DFMT_UNKNOWN,	//format
			D3DPOOL_DEFAULT, //memory pool
			D3DX_DEFAULT, //filtering
			D3DX_DEFAULT, //mipmap filtering
			0xff000000,	//colorkey
			NULL,		//source info
			NULL,		//palette
			&TexOver)))
		{
			//looks like it failed, which is a bad thing
			logfile.LogtoFile("SPR - failed to load mouse over sprite texture.\n");
			return GUI_BAD;
		}
	}


	//try setting up the vertex buffer
	SPRITE_CUSTOMVERTEX vertexarray[] = 
	{
		//the the first vertex - the upper left hand corner of the sprites
		{Position.x, Position.y, 0.0f, Color, 0.0f, 0.0f},

		//top right vertex
		{(Position.x + Width), Position.y, 0.0f, Color, 1.0f, 0.0f},
		
		//bottom right vertex
		{(Position.x + Width), (Position.y - Height), 0.0f, Color, 1.0f, 1.0f},

		//bottom left vertex
		{Position.x, (Position.y - Height), 0.0f, Color, 0.0f, 1.0f}
	};

	//actually create the vertex buffer
	if(FAILED(Device->CreateVertexBuffer(4*sizeof(SPRITE_CUSTOMVERTEX), D3DUSAGE_WRITEONLY, 
		SPRITE_D3DFVF_CUSTOMVERTEX, D3DPOOL_DEFAULT, &VertexBuffer)))
	{
		//uh-oh, it failed!
		logfile.LogtoFile("SPR - failed to create vertex buffer.\n");
		return GUI_BAD;
	}

	//now fill in the vertex buffer
	VOID* pVertices;
	if(FAILED(VertexBuffer->Lock(0, sizeof(vertexarray), (BYTE**)&pVertices, 0)))
	{
		//hmm, this can't be good!
		logfile.LogtoFile("SPR - failed to lock sprite vertex buffer.\n");
		return GUI_BAD;
	}

	//copy the vertices to the vertex buffer
	memcpy( pVertices, vertexarray, sizeof(vertexarray) );

	//unlock the vertex buffer
	VertexBuffer->Unlock();

	//make sure the sprite is set to the right position
	SetSpritePosition(Position.x, Position.y);

	//everything must have gone ok, so return so
	return GUI_OK;

}

//--------------------------------------------------//
// Name: RenderSprite()
// Desc: draws the sprite
//--------------------------------------------------//
int Sprite::RenderSprite()
{
	//set up the 2d camera projections
	D3DXMATRIX matOrtho; 
    D3DXMATRIX matIdentity;
	D3DMATRIX *tempproj;
	D3DMATRIX *tempworld;
	D3DMATRIX *tempview;

	//store the previous transformations so they can be reset when done
	Device->GetTransform(D3DTS_PROJECTION, tempproj);
	Device->GetTransform(D3DTS_WORLD, tempworld);
	Device->GetTransform(D3DTS_VIEW, tempview);
    
    //Setup the orthogonal projection matrix and the default world/view matrix
    D3DXMatrixOrthoLH(&matOrtho, (float)ScreenWidth, (float)ScreenHeight, 0.0f, 1.0f);
    D3DXMatrixIdentity(&matIdentity);

    Device->SetTransform(D3DTS_PROJECTION, &matOrtho);
    Device->SetTransform(D3DTS_WORLD, &matIdentity);
    Device->SetTransform(D3DTS_VIEW, &matIdentity);

    //Make sure that the z-buffer and lighting are disabled
    Device->SetRenderState(D3DRS_ZENABLE, D3DZB_FALSE);
    Device->SetRenderState(D3DRS_LIGHTING, FALSE);

	//set the position of the sprite
	//SetSpritePosition(Position.x, Position.y);

	//do the actual rendering of the sprite
	Device->SetStreamSource(0, VertexBuffer, sizeof(SPRITE_CUSTOMVERTEX));
    Device->SetVertexShader(SPRITE_D3DFVF_CUSTOMVERTEX);

	if(MouseOver == false)
	{
		if(Tex != NULL)
		{
			Device->SetTexture(0, Tex);
			//Device->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_MODULATE);
		}
	}

	if(MouseOver == true)
			Device->SetTexture(0, TexOver);


	if(FAILED(Device->DrawPrimitive(D3DPT_TRIANGLEFAN, 0, 2)))
	{	
		//it failed for some reason
		logfile.LogtoFile("SPR - failed to render sprite.\n");
		return GUI_BAD;
	}
	if(MouseOver == false)
		Device->SetTexture(0, Tex);
	else
		Device->SetTexture(0, TexOver);

	if(FAILED(Device->DrawPrimitive(D3DPT_TRIANGLEFAN, 0, 2)))
	{	
		//it failed for some reason
		logfile.LogtoFile("SPR - failed to render sprite.\n");
		return GUI_BAD;
	}


	//turn the lighting and z-buffer back on
	Device->SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);
    Device->SetRenderState(D3DRS_LIGHTING, TRUE);
	
	//set the transforms back to what they were
	Device->SetTransform(D3DTS_PROJECTION, tempproj);
    Device->SetTransform(D3DTS_WORLD, tempworld);
    Device->SetTransform(D3DTS_VIEW, tempview);


	//return ok
	return GUI_OK;
}

//--------------------------------------------------//
// Name: Destroyprite()
// Desc: releases all the stuff that was allocated
//--------------------------------------------------//
void Sprite::DestroySprite()
{
	//do the safe release thingy
	SAFE_RELEASE(Tex);
	SAFE_RELEASE(TexOver);
	SAFE_RELEASE(VertexBuffer);

}

//--------------------------------------------------//
// Name: SetSpritePosition()
// Desc: sets the position of the sprite
//--------------------------------------------------//
void Sprite::SetSpritePosition(float x, float y)
{
    //x and y specify the top left corner of the panel in screen coordinates
    D3DXMATRIX matMove;
	D3DXVECTOR2 pos = D3DXVECTOR2(0.0f, 0.0f);

    pos.x = (-ScreenWidth/2) + x;
    pos.y = (-ScreenHeight/2) - y;
	//x -= (ScreenWidth/2);
	//y -= (ScreenHeight/2);

    D3DXMatrixTranslation(&matMove, pos.x, pos.y, 0.0f);
    
    Device->SetTransform(D3DTS_WORLD, &matMove);


}

//--------------------------------------------------//
// Name: IsMouseOver()
// Desc: tests if the mouse is over the sprite
//--------------------------------------------------//
bool Sprite::IsMouseOver(int x, int y)
{
	//test if the mouse is over the sprite
	if( (x >= Position.x) && (x <= (Position.x + Width)) && (y >= Position.y) && (y <= (Position.y + Height)))
	{	
		MouseOver = true;
		return true;
	} else {
		MouseOver = false;
		return false;
	}
}*/