//----------------------------------------//
// milkshape.h - header file for the Milkshape model loading code
//
//----------------------------------------//


//prevent multiple inclusions
#ifndef _MILKSHAPE_H_
#define _MILKSHAPE_H_

//include the engine definitions
#include "enginedef.h"
#include "d3d.h"

//the return types for the models
const int MSMODEL_FAIL			= 0;	//it was bad
const int MSMODEL_SUCCESS		= 1;	//it was good

//the custom vertex definition for milkshape models
struct MODELVERTEX
{
	D3DXVECTOR3 position;	//the 3D position of the vertex
	D3DXVECTOR3 normal;		//the surface normal for the vertex
	FLOAT       tu, tv;		// The texture coordinates
};

//define the FVF, which describes the custom vertex structure
#define D3DFVF_MODELVERTEX (D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_TEX1)

//the intermediate classes - they are used to help extract the data into the 
//vertex buffer.  It would be a mess to load it in otherwise

//the intermediate vertex normal face
class INTERMEDIATE_VERT
{
public:
	float	x,y,z,	//coordinates
			u,v;	//texture coords
};


//the intermediate normal class
class INTERMEDIATE_NORMAL
{
public:
	float nx,ny,nz;		//normal

};


//the intermediate face class
class INTERMEDIATE_FACE
{
public:
		int a,b,c,		//vertex index for the verts
		na,nb,nc;		//normal index for the verts
};


//the intermediate mesh class - used to store the data so that it can be put into the
//vertex buffer
class INTERMEDIATE_GROUP
{
public:
	char *szName[64];			//name of this mesh group
	int iMaterial;				//material index
	int iVerts;					//vertex count
	INTERMEDIATE_VERT* pVerts;	//vertex array
	int iNormals;				//normal count
	INTERMEDIATE_NORMAL* pNormals;	//normal array
	int iFaces;					//face count
	INTERMEDIATE_FACE* pFaces;		//face array
};


//the final mesh group for the model
class FINAL_GROUP
{
public:
	LPDIRECT3DVERTEXBUFFER8	pVB;			//D3D8 Vertex buffer
	int	iPrimitives;	// # of primitives to render from the VB
	int	iMaterial;		//material index
};


//the final material group for the model
class FINAL_MATERIAL
{
public:
	D3DMATERIAL8 Mat;		// D3D8 Material structure
	LPDIRECT3DTEXTURE8 pTex1;	//first texture
	LPDIRECT3DTEXTURE8 pTex2;	//second texture
	float fAlpha;				//alpha opacity
	bool bRef;					//reflective mapping?
	bool alphablend;			//alpha blend or not?
};

//the generic material that is used if one is not specified
static D3DMATERIAL8 g_GenericMaterial = 
{
	{1.0f, 1.0f, 1.0f, 1.0f},
	{0.0f, 0.0f, 0.0f, 0.0f},
	{1.0f, 1.0f, 1.0f, 1.0f},
	{0.0f, 0.0f, 0.0f, 0.0f},
	0
};

//the model class
class MODEL
{
public:
	MODEL();
	~MODEL();

	// Mesh groups
	int	NumGroups;				//mesh group count
	FINAL_GROUP *PointerGroups;	//mesh groups

	// Materials
	int	NumMaterials;	//material count
	FINAL_MATERIAL*	PointerMaterials;	//materials

	D3DXMATRIX matModel;	//the model's matrix
	D3DXVECTOR3	Position;	//the model's position
	D3DXVECTOR3 Velocity;	//the model's velocity

	//loads the file name and changes it
	HRESULT GetPathAndFile(char * szFilename, char * szPath, char * szFile);

	//creates the model and loads it
	HRESULT CreateModelFromFile(LPDIRECT3DDEVICE8 device, char * szFilename, float fScale);

	//does the actual loading
	HRESULT LoadModel(LPDIRECT3DDEVICE8 device, char * szFilename, float fScale);

	//render model
	HRESULT RenderModel(LPDIRECT3DDEVICE8 device);
	
	//note: all the cleanup is done by the destructor - no need to call a
	//cleanup function :)
};

#endif //for _MILKSHAPE_H_